; main.s
; Runs on LM4F120
; Provide functions that initialize a GPIO pin and turn it on and off.
; Use bit-banded I/O.
; Daniel Valvano
; October 1, 2012

;  This example accompanies the book
;   "Embedded Systems: Introduction to Arm Cortex M Microcontrollers"
;   ISBN: 978-1469998749, Jonathan Valvano, copyright (c) 2012
;   Example 2.1, Program 2.7, Figure 2.25
;
;Copyright 2012 by Jonathan W. Valvano, valvano@mail.utexas.edu
;   You may use, edit, run or distribute this file
;   as long as the above copyright notice remains
;THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
;OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
;MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
;VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
;OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
;For more information about my classes, my research, and my books, see
;http://users.ece.utexas.edu/~valvano/

; solid state relay connected to PF2

GPIO_PORTF_DATA_R  EQU 0x400253FC
GPIO_PORTF_DIR_R   EQU 0x40025400
GPIO_PORTF_AFSEL_R EQU 0x40025420
GPIO_PORTF_PUR_R   EQU 0x40025510
GPIO_PORTF_DEN_R   EQU 0x4002551C
GPIO_PORTF_AMSEL_R EQU 0x40025528
GPIO_PORTF_PCTL_R  EQU 0x4002552C
PF2                EQU 0x40025010
PF4                EQU 0x40025040
SYSCTL_RCGC2_R     EQU 0x400FE108
SYSCTL_RCGC2_GPIOF EQU 0x00000020   ; port F Clock Gating Control

        AREA    |.text|, CODE, READONLY, ALIGN=2
        THUMB
        EXPORT  Start
; This is an extremely simple test program to demonstrate that the SSR
; can turn on and off.  Button de-bouncing is not addressed.
; built-in negative logic switch connected to PF4
Start
    BL  SSR_Init                    ; initialize PF2 and make it output
    ; set direction register
    LDR R1, =GPIO_PORTF_DIR_R       ; R1 = &GPIO_PORTF_DIR_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x10               ; R0 = R0&~0x10 (make PF4 input; PF4 built-in button #1)
    STR R0, [R1]                    ; [R1] = R0
    ; regular port function
    LDR R1, =GPIO_PORTF_AFSEL_R     ; R1 = &GPIO_PORTF_AFSEL_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x10               ; R0 = R0&~0x10 (disable alt funct on PF4)
    STR R0, [R1]                    ; [R1] = R0
    ; pull-up resistors on switch pins
    LDR R1, =GPIO_PORTF_PUR_R       ; R1 = &GPIO_PORTF_PUR_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x10               ; R0 = R0|0x10 (enable pull-up on PF4)
    STR R0, [R1]                    ; [R1] = R0
    ; enable digital port
    LDR R1, =GPIO_PORTF_DEN_R       ; R1 = &GPIO_PORTF_DEN_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x10               ; R0 = R0|0x10 (enable digital I/O on PF4)
    STR R0, [R1]                    ; [R1] = R0
    ; configure as GPIO
    LDR R1, =GPIO_PORTF_PCTL_R      ; R1 = &GPIO_PORTF_PCTL_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x000F0000         ; R0 = R0&~0x000F0000 (clear port control field for PF4)
    ADD R0, R0, #0x00000000         ; R0 = R0+0x00000000 (configure PF4 as GPIO)
    STR R0, [R1]                    ; [R1] = R0
    LDR R4, =PF4                    ; R4 = &PF4
loop                                ; in this loop, the appliance (PF2) toggles when the switch is released
    BL  SSR_On
waitforpress1                       ; proceed only when the button is pressed
    LDR R0, [R4]                    ; R0 = [R4] (read status of PF4)
    CMP R0, #0x10                   ; R0 == 0x10?
    BEQ waitforpress1               ; if so, spin
waitforrelease1                     ; proceed only when the button is released
    LDR R0, [R4]                    ; R0 = [R4] (read status of PF4)
    CMP R0, #0x10                   ; R0 != 0x10?
    BNE waitforrelease1             ; if so, spin
    BL  SSR_Off
waitforpress2                       ; proceed only when the button is pressed
    LDR R0, [R4]                    ; R0 = [R4] (read status of PF4)
    CMP R0, #0x10                   ; R0 == 0x10?
    BEQ waitforpress2               ; if so, spin
waitforrelease2                     ; proceed only when the button is released
    LDR R0, [R4]                    ; R0 = [R4] (read status of PF4)
    CMP R0, #0x10                   ; R0 != 0x10?
    BNE waitforrelease2             ; if so, spin
    B   loop

;------------SSR_Init------------
; Make PF2 an output, enable digital I/O, ensure alt. functions off.
; Input: none
; Output: none
; Modifies: R0, R1
SSR_Init
    ; activate clock for Port F
    LDR R1, =SYSCTL_RCGC2_R         ; R1 = &SYSCTL_RCGC2_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #SYSCTL_RCGC2_GPIOF ; R0 = R0|SYSCTL_RCGC2_GPIOF
    STR R0, [R1]                    ; [R1] = R0
    NOP
    NOP                             ; allow time to finish activating
    ; set direction register
    LDR R1, =GPIO_PORTF_DIR_R       ; R1 = &GPIO_PORTF_DIR_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x04               ; R0 = R0|0x04 (make PF2 output)
    STR R0, [R1]                    ; [R1] = R0
    ; regular port function
    LDR R1, =GPIO_PORTF_AFSEL_R     ; R1 = &GPIO_PORTF_AFSEL_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x04               ; R0 = R0&~0x04 (disable alt funct on PF2)
    STR R0, [R1]                    ; [R1] = R0
    ; enable digital port
    LDR R1, =GPIO_PORTF_DEN_R       ; R1 = &GPIO_PORTF_DEN_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x04               ; R0 = R0|0x04 (enable digital I/O on PF2)
    STR R0, [R1]                    ; [R1] = R0
    ; configure as GPIO
    LDR R1, =GPIO_PORTF_PCTL_R      ; R1 = &GPIO_PORTF_PCTL_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x00000F00         ; R0 = R0&~0x00000F00 (clear port control field for PF2)
    ADD R0, R0, #0x00000000         ; R0 = R0+0x00000000 (configure PF2 as GPIO)
    STR R0, [R1]                    ; [R1] = R0
    ; disable analog functionality
    LDR R1, =GPIO_PORTF_AMSEL_R     ; R1 = &GPIO_PORTF_AMSEL_R
    MOV R0, #0                      ; R0 = 0 (disable analog functionality on PF)
    STR R0, [R1]                    ; [R1] = R0
    BX  LR                          ; return

;------------SSR_On------------
; Make PF2 high.
; Input: none
; Output: none
; Modifies: R0, R1
SSR_On
    LDR R1, =PF2                    ; R1 = &PF2
    MOV R0, #0x04                   ; R0 = 0x04 (turn on the appliance)
    STR R0, [R1]                    ; [R1] = R0, write to PF2
    BX  LR                          ; return

;------------SSR_Off------------
; Make PF2 low.
; Input: none
; Output: none
; Modifies: R0, R1
SSR_Off
    LDR R1, =PF2                    ; R1 = &PF2
    MOV R0, #0x00                   ; R0 = 0x00 (turn off the appliance)
    STR R0, [R1]                    ; [R1] = R0, write to PF2
    BX  LR                          ; return

    ALIGN                           ; make sure the end of this section is aligned
    END                             ; end of file
